#include <math.h>
#include "types.h"
#include "vector.h"

/*****************************************************************************
 3D Vector Code

 All vectors are centered on origin (0,0,0).
*****************************************************************************/

typedef struct
{
    float   data[4][4];
} MATRIX;

/*
 * Rotates a vector
 */

void Vec3D_Rotate(VECTOR3D *in, float a_x, float a_y, float a_z)
{
    MATRIX      m;
    VECTOR3D    out;


        m.data[0][0] = cos(DEG2RAD(a_y)) * cos(DEG2RAD(a_z));
        m.data[0][1] = -cos(DEG2RAD(a_y)) * sin(DEG2RAD(a_z));
        m.data[0][2] = sin(DEG2RAD(a_y));
        m.data[0][3] = 0;
        m.data[1][0] = sin(DEG2RAD(a_x)) * sin(DEG2RAD(a_y)) * cos(DEG2RAD(a_z)) + cos(DEG2RAD(a_x)) * sin(DEG2RAD(a_z));
        m.data[1][1] = -sin(DEG2RAD(a_x)) * sin(DEG2RAD(a_y)) * sin(DEG2RAD(a_z)) + cos(DEG2RAD(a_x)) * cos(DEG2RAD(a_z));
        m.data[1][2] = -sin(DEG2RAD(a_x)) * cos(DEG2RAD(a_y));
        m.data[1][3] = 0;
        m.data[2][0] = -cos(DEG2RAD(a_x)) * sin(DEG2RAD(a_y)) * cos(DEG2RAD(a_z)) + sin(DEG2RAD(a_x)) * sin(DEG2RAD(a_z));
        m.data[2][1] = cos(DEG2RAD(a_x)) * sin(DEG2RAD(a_y)) * sin(DEG2RAD(a_z)) + sin(DEG2RAD(a_x)) * cos(DEG2RAD(a_z));
        m.data[2][2] = cos(DEG2RAD(a_x)) * cos(DEG2RAD(a_y));
        m.data[2][3] = 0;
        m.data[3][0] = 0;
        m.data[3][1] = 0;
        m.data[3][2] = 0;
        m.data[3][3] = 0;

    out.x = in->x * m.data[0][0] + in->y * m.data[1][0] + in->z * m.data[2][0] + 1 * m.data[3][0];
    out.y = in->x * m.data[0][1] + in->y * m.data[1][1] + in->z * m.data[2][1] + 1 * m.data[3][1];
    out.z = in->x * m.data[0][2] + in->y * m.data[1][2] + in->z * m.data[2][2] + 1 * m.data[3][2];

    *in = out;
}

/*
 * Returns the dot product between 2 vectors. No assumptions are made about
 * any of the vectors (neither needs to be a unit vector.) This is useful
 * where only the sign of a dot product is needed and no unit vectors are
 * handy.
 */

float Vec3D_RawDotProduct(VECTOR3D *u, VECTOR3D *v)
{
    return (u->x * v->x + u->y * v->y + u->z * v->z);
}
